/*
 * Copyright (C) 2011 Marco Trevisan (Treviño) <mail@3v1n0.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 */

package net3v1n0.dev.atherosreloader;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.method.LinkMovementMethod;
import android.text.util.Linkify;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.TextView;
import android.widget.Toast;
import android.net.wifi.WifiManager;
import android.content.DialogInterface;

public class MainClass extends Activity {
	private static final int DIALOG_WIFI_ACTIVE = 1;
	private static final int DIALOG_ABOUT = 2;
	private WifiManager wm;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        this.wm = (WifiManager) getSystemService(WIFI_SERVICE);

        setupUI();
    }

    @Override
    public void onResume() {
    	super.onResume();
    	isRooted();
    }

    @Override
    public Dialog onCreateDialog(int id) {
    	switch (id) {
    	case DIALOG_WIFI_ACTIVE:
    		LayoutInflater checkInflater = LayoutInflater.from(this);
    		View lt = checkInflater.inflate(R.layout.checkbox, null);
    	    final CheckBox cb = (CheckBox)lt.findViewById(R.id.restart_wifi_ck);
    	    cb.setChecked(true);

    	    return new AlertDialog.Builder(this)
	        .setMessage(R.string.wifi_on)
	        .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
	            public void onClick(DialogInterface dialog, int whichButton) {
	            	restartWifiUI(cb.isChecked());
	            }
	        })
	        .setNegativeButton(android.R.string.cancel, null)
	        .setView(lt)
	        .create();
    	case DIALOG_ABOUT:
    		final TextView message = new TextView(this);
    		String txt = getString(R.string.about_msg).replaceAll("__VERSION__",
    			         getString(R.string.app_version));
    		final SpannableString s = new SpannableString(txt);
    		Linkify.addLinks(s, Linkify.WEB_URLS|Linkify.EMAIL_ADDRESSES);
    		message.setText(s);
    		message.setMovementMethod(LinkMovementMethod.getInstance());
    		message.setTextSize(message.getTextSize()*2);

    		return new AlertDialog.Builder(this)
    		.setTitle(R.string.about)
	        .setView(message)
	        .setPositiveButton(android.R.string.ok, null)
	        .create();
    	}

    	return null;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	if (item.getItemId() == R.id.menu_info) {
    		MainClass.this.showDialog(DIALOG_ABOUT);
    		return true;
    	}
		return false;
    }

    private void setupUI() {
    	final Button action_button = (Button) findViewById(R.id.button_reload);
    	action_button.setOnClickListener(new OnClickListener() {

    	    public void onClick(View v) {
    	    	if (!MainClass.this.wm.isWifiEnabled()) {
    	    		MainClass.this.restartWifiUI(false);
    	    	} else {
    	    		MainClass.this.showDialog(DIALOG_WIFI_ACTIVE);
    	    	}
    	    }
    	});
    }

    private void runRootCommands(List<String> cmds) throws IOException, InterruptedException {
    	Process root = Runtime.getRuntime().exec("su");
		OutputStream os = root.getOutputStream();

		if (cmds == null)
			cmds = new ArrayList<String>();

		cmds.add("exit");
		for (String cmd : cmds) {
			os.write(cmd.getBytes());
			os.write(new String("\n").getBytes());
		}

		os.flush();
		cmds.remove(cmds.size()-1);
		root.waitFor();

		if (root.exitValue() != 255) {
			root.destroy();
		} else {
			throw new InterruptedException("No root available");
		}
    }

    private boolean isRooted() {
    	boolean rooted = false;

    	try {
    		runRootCommands(null);
    		rooted = true;
		} catch (Exception e) {
			Toast.makeText(this, R.string.error_root_exit, Toast.LENGTH_SHORT).show();
			this.finish();
		}

		return rooted;
    }

    private boolean restartWifi(boolean reconnect) {
    	boolean was_connected = false;

    	if (wm.isWifiEnabled()) {
    		wm.disconnect();
    		wm.setWifiEnabled(false);
    		was_connected = true;
    	}

    	try {
			List<String> cmd_list = new ArrayList<String>();
			cmd_list.add("rmmod ar6000");
			cmd_list.add("insmod /system/wifi/ar6000.ko");
			runRootCommands(cmd_list);
		} catch (Exception e) {
			return false;
		}

		if (reconnect && was_connected) {
			try {
				final int MAX_RETRIES = 3;
				int retries = 0;
				boolean enabled;

				Thread.sleep(500);
				enabled = wm.setWifiEnabled(true);

				while (!enabled && retries < MAX_RETRIES) {
					retries++;
					Thread.sleep(250);
					enabled = wm.setWifiEnabled(true);
				}
				wm.reconnect();
			} catch (InterruptedException e) {}
		}

    	return true;
    }

    private boolean restartWifiUI(boolean reconnect) {
    	int msg;
    	boolean ret;

		if (restartWifi(reconnect)) {
			msg = R.string.atheros_restart_ok;
			ret = true;
		} else {
			msg = R.string.atheros_restart_ko;
			ret = false;
		}

		Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();

		return ret;
    }
}